package edu.fauser.netlab;

import org.glassfish.jersey.client.authentication.HttpAuthenticationFeature;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.GenericType;
import javax.ws.rs.core.Response;
import java.util.List;

public class ClientColori {
    private static WebTarget target;
    public static void main(String[] args) {

        // Impostazione delle credenziali per l'accesso alle API in modalità UNIVERSAL
        // In questa modalità il client attiva l'autenticazione BASIC oppure DIGEST
        // in base al contenuto dell'header WWW-Authenticate.

        HttpAuthenticationFeature auth = HttpAuthenticationFeature.universalBuilder()
                .credentials("utente1-rest", "123456").build();

        // Costruzione di un nuovo oggetto client in grado di gestire automaticamente
        // la fase di autenticazione HTTP su richiesta del server
        Client client = ClientBuilder.newClient().register(auth);

        // Definizione dell'URI della risorsa obiettivo ("target")
        // In questo esempio la variabile "target" è condivisa
        // da tutte le funzioni di elaborazione

        target = client.target("http://localhost:8080/server-colori-http/api").path("colori");

        // Stampa del contenuto iniziale della mappa memorizzata nel server
        stampaElencoColori();

        // Aggiunge un nuovo colore alla mappa
        creaNuovoColore("Blu", "0,0,255", "#0000ff");

        // Aggiornamento di un dato colore ...
        modificaColore(3, "0,0,254", "#0000fe");

        // ... E relativa stampa su schermo
        stampaSingoloColore(3);
    }

    private static void stampaElencoColori() {
        // si esegue una richiesta GET sulla risorsa target definita nel main (/api/colori)
        // e si memorizza la risposta
        Response r = target.request().get();

        // Informazioni sullo stato della risposta HTTP
        System.out.format("stampaElencoColori - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());

        if (r.getStatus() == 200) {
            // Legge l'informazione contenuta nel body della risposta HTTP (entità in formato JSON)
            // e la trasforma in una lista di oggetti "ColoreBreve"

            List<ColoreBreve> colori = r.readEntity(new GenericType<List<ColoreBreve>>() {});
            for (var c : colori) {
                System.out.format("Colore %s (id: %d)%n", c.getNome(), c.getId());
            }
        }
    }
    private static void creaNuovoColore(String nome, String rgb, String hex) {
        // Crea un nuovo oggetto "Colore" a partire dai dati del nuovo colore
        // L'attributo id è tralasciato in quanto determinato automaticamente dal server
        Colore c = new Colore(null,nome, rgb, hex);

        // Esegue una  richiesta POST sulla risorsa target (/api/colori)
        // Il body della richiesta include il colore da creare in formato JSON
        Response r = target.request().post(Entity.json(c));

        System.out.format("creaNuovoColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
        if (r.getStatus() == 201) {
            System.out.format("Location: %s%n", r.getLocation());
        }
    }

    private static void modificaColore(Integer idColore, String rgb, String hex) {
        // Aggiornamento dei codici di colore con il metodo PUT.
        // Prima fase: si acquisisce la descrizione iniziale del colore

        Response r = target.path(idColore.toString()).request().get();

        if (r.getStatus() != 200) {
            System.out.format("modificaColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
            return;
        }
        // Seconda fase: trasformazione dell'entità JSON in un oggetto Java
        // e modifica dei codici RGB e HEX
        Colore c = r.readEntity(Colore.class);
        c.setRgb(rgb);
        c.setHex(hex);

        // Terza fase: invio di una richiesta PUT per aggiornare l'intero colore
        r = target.path(idColore.toString()).request().put(Entity.json(c));
        System.out.format("modificaColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
    }
    private static void stampaSingoloColore(Integer idColore) {
        // Acquisizione della descrizione del colore dalla risorsa target /api/colori/{idColore}
        Response r = target.path(idColore.toString()).request().get();

        System.out.format("stampaSingoloColore - Status: %d (%s)%n", r.getStatus(), r.getStatusInfo().getFamily().name());
        if (r.getStatus() == 200) {
            Colore c = r.readEntity(Colore.class);
            System.out.printf("Id: %d  NomeColore: %s   Rgb: %s  Hex: %s%n", c.getId(), c.getNome(), c.getRgb(), c.getHex());
        }
    }
}